.TH "Simple Virtual Machine - Plugin coroutine" 7 "2022-02-19"
.SH SYNOPSIS
.B svm_plugin_coroutine
An implementation of coroutines for SVM.
.SH DESCRIPTION
This plugin enables the support of coroutines.
Several coroutines can be created and used at a given point in programs.
.P
The plugin uses processor flags starting with 'coroutine:' (by default) to handle the coroutines and all flags with this prefix shall not be used in applications using coroutines.
This prefix can be changed through the -p plugin option.
.SH INVOKATION
This plugin can be added to the Simple Virtual Machine using one of these directives:
.SS Local
.nf

LOCAL PLUGIN "svmplugincoroutine/libsvmcoroutine.so"

.fi
when compiled/installed locally and should be accessible from the application directory.
.SS Machine and System
.nf

PLUGIN "===PLUGINLIB==="

.fi
when installed with auto-tools and should be accessible from the Simple Virtual Machine installation directory or from system directories. (Recommended)
.SS Absolute path
.nf

PLUGIN "===PLUGINLIBDIR===/===PLUGINLIB==="

.fi
when installed with auto-tools and should be accessible from an absolute path.
.SH CONTENT
.SS OPTION coroutine.prefix -p STR
Changes the coroutines flag prefix.
.SS INSTRUCTION coroutine.call STR:name [ STR SYM ]:function PTR:parameters
Create a named coroutine. The coroutine is placed in the background.
.P
This instruction raises a FAILURE interruption when a coroutine with the same name already exists.
.SS INSTRUCTION coroutine.run STR:name
Resume execution of the named coroutine.
.P
This instruction raises a FAILURE interruption when a coroutine with the given name does not exist.
.SS INSTRUCTION coroutine.yield
Pause execution of the current coroutine and resume execution of its caller.
.P
This instruction does nothing when called outside a coroutine.
.SS INSTRUCTION coroutine.return STR:name
Destroy the named coroutine in the background.
.P
This instruction raises a FAILURE interruption when a coroutine with the given name does not exist, or when this instruction attempts to return the coroutine from itself.
.P
A coroutine can return by itself using the return instruction.
Any subsequent call to such coroutine will fail after it returns.
.SH EXAMPLES
.SS Fibonacci generator
.nf
#!===SVMBIN===
LOG
PLUGIN "svmint.so"
PLUGIN "svmcom.so"
PLUGIN "===PLUGINLIB==="
ARGUMENT INT nb
PROCESS "fibonacci"
	CODE "main" INLINE
		:memory (PTR, INT)/p, INT/i
		[ 0 ] -> i
		:coroutine.call "fib" "fib" p
	:label loop
		:coroutine.run "fib"
		:com.message @(p/1)
		:shift &i
		:goto loop :when @&i IN &0*@&nb
		:coroutine.return "fib"
		:shutdown

	:label fib
		:memory INT, INT -> &P
		0 -> (P/1)
		:coroutine.yield
		:shift (P/1)
		:coroutine.yield
		[ 0 , 1 ] -> @&P
	:label loop_fib
		:int.add @&P -> (P/1)
		@(@&P/1) -> (@&P/0)
		@(P/1) -> (@&P/1)
		:coroutine.yield
		:goto loop_fib
	END
	MEMORY nb
END
.fi
.SS A complete example
.nf
#!===SVMBIN===
LOG
PLUGIN "svmint.so"
PLUGIN "svmcom.so"
PLUGIN "===PLUGINLIB==="
ARGUMENT INT nb
PROCESS "coroutine"
	CODE "main" INLINE
		:memory INT/i, INT*@&nb/t
		0 -> &i
	:label init
		:int.sub @&nb @&i -> (t/@&i)
		:shift &i
		:goto init :when @&i IN t
		:memory (PTR, PTR, STR, INT, INT)/p
		[ , t , "sum" , 0 , ] -> p
		:call accumulate p
		:com.message "Result=" @(p/4) 
		:com.message "Explanation:"
		:coroutine.call "accumulate" "accumulate" p
		:memory (PTR, PTR, STR)/pp
		[ , p , ] -> pp
		:coroutine.call "parity" "parity" pp
	:label loop
		:coroutine.run "parity"
		:shutdown :unless (@&p/5) DEFINED
		:com.message @(@&p/5) " (" @(pp/2) ")"
		:goto loop

	:label accumulate # PTR, PTR:array of int, STR/SYM:function, INT:start, INT:result
		:memory INT, INT, PTR, INT, INT, INT -> &P
		[ @(P/3) , 0 ] -> &@&P*2
	:label accumulate_loop
		@&@&P -> (@&P/3)
		@(@(P/1)/@(@&P/1)) -> (@&P/4)
		:call @(P/2) (&@&P+2)*4
		@(@&P/5) -> &@&P
		:shift (@&P/1)
		:goto accumulate_loop :when @(@&P/1) IN @(P/1)
		@&@&P -> (P/4)
		:return
	
	:label sum # PTR, INT:previous, INT:value, INT:result
		:int.add @(P/1) @(P/2) -> (P/3)
		:coroutine.yield
		:return

	:label parity # PTR, PTR:function_params, STR:result
		:memory INT, BLN -> &P
	:label parity_loop
		:coroutine.run "accumulate"
		:return :unless (@&@(P/1)/5) DEFINED
		:int.mod @(@&@(P/1)/5) 2 -> &@&P
		:int.cmp @&@&P = 0 -> (@&P/1)
		:call even P :when @(@&P/1) TRUE
		:call odd P :unless @(@&P/1) TRUE
		:coroutine.yield
		:goto parity_loop
	:label even
		"even" -> (P/2)
		:return
	:label odd
		"odd" -> (P/2)
		:return
	END
	MEMORY nb
END
.fi
.SH SEE ALSO
.BR svm (1)
for help on the Simple Virtual Machine.
.SH AUTHOR
This plugin has been written by Julien BRUGUIER and is maintained by Julien BRUGUIER <projet.svm@pappy.tf>.
